@@ -25,12 +25,16 @@ from typing import Union, List, Tuple, Callable, Dict
 from dataclasses import dataclass
 from tqdm import tqdm
 from roguewave import logger
-from .remote_resources import RemoteResourceS3, \
-    RemoteResourceHTTPS, RemoteResource, RemoteResourceLocal
+from .remote_resources import (
+    RemoteResourceS3,
+    RemoteResourceHTTPS,
+    RemoteResource,
+    RemoteResourceLocal,
+)
 from .exceptions import _RemoteResourceUriNotFound
 from json import dumps, load
 
-TEMPORARY_DIRECTORY = '~/temporary_roguewave_files/filecache/'
+TEMPORARY_DIRECTORY = "~/temporary_roguewave_files/filecache/"
 CACHE_SIZE_GB = 5
 MAXIMUM_NUMBER_OF_WORKERS = 10
 KILOBYTE = 1000
@@ -82,17 +86,18 @@ class FileCache:
     ...
     """
 
-    CACHE_FILE_PREFIX = 'cachefile_'
-    CACHE_FILE_POSTFIX = '_cachefile'
+    CACHE_FILE_PREFIX = "cachefile_"
+    CACHE_FILE_POSTFIX = "_cachefile"
 
-    def __init__(self,
-                 path: str = TEMPORARY_DIRECTORY,
-                 size_GB: Union[float, int] = CACHE_SIZE_GB,
-                 do_cache_eviction_on_startup: bool = False,
-                 resources: List[RemoteResource] = None,
-                 parallel=True,
-                 allow_for_missing_files=False,
-                 ):
+    def __init__(
+        self,
+        path: str = TEMPORARY_DIRECTORY,
+        size_GB: Union[float, int] = CACHE_SIZE_GB,
+        do_cache_eviction_on_startup: bool = False,
+        resources: List[RemoteResource] = None,
+        parallel=True,
+        allow_for_missing_files=False,
+    ):
         """
         Initialize Cache
         :param path: path to store cache. If path does not exist it will be
@@ -118,7 +123,7 @@ class FileCache:
         self.config = {
             "size_gb": size_GB,
             "parallel": parallel,
-            "allow_for_missing_files":allow_for_missing_files
+            "allow_for_missing_files": allow_for_missing_files,
         }
         if self.config_exists():
             self.config |= self.load_config()
@@ -137,25 +142,27 @@ class FileCache:
         self._initialize_cache(do_cache_eviction_on_startup)
 
         # Post processing and validation functions
-        self.directives = {"validate": {}, "postprocess":{}}
+        self.directives = {"validate": {}, "postprocess": {}}
 
         # download resources
         if resources is None:
-            self.resources = [RemoteResourceS3(),
-                              RemoteResourceHTTPS(),
-                              RemoteResourceLocal()]
+            self.resources = [
+                RemoteResourceS3(),
+                RemoteResourceHTTPS(),
+                RemoteResourceLocal(),
+            ]
         else:
             self.resources = resources
 
     @property
     def config_name(self) -> str:
-        return os.path.join(self.path,'file_cache_config.json')
+        return os.path.join(self.path, "file_cache_config.json")
 
-    def config_exists(self)->bool:
+    def config_exists(self) -> bool:
         return os.path.exists(self.config_name)
 
-    def load_config(self)->Dict:
-        with open( self.config_name, 'rb' ) as fp:
+    def load_config(self) -> Dict:
+        with open(self.config_name, "rb") as fp:
             return load(fp)
 
     def _update_config(self, key, value, write=True):
@@ -164,72 +171,66 @@ class FileCache:
             self._write_config()
 
     def _write_config(self):
-        with open( os.path.join(
-                self.path,'file_cache_config.json'),'wt' ) as fp:
-            fp.write(dumps(self.config,indent=4))
+        with open(os.path.join(self.path, "file_cache_config.json"), "wt") as fp:
+            fp.write(dumps(self.config, indent=4))
 
     @property
-    def max_size(self)->int:
-        return self.config['size_gb']
+    def max_size(self) -> int:
+        return self.config["size_gb"]
 
     @max_size.setter
-    def max_size(self,size_gb:float):
-        self._update_config('size_gb',size_gb)
+    def max_size(self, size_gb: float):
+        self._update_config("size_gb", size_gb)
 
     @property
-    def max_size_bytes(self)->int:
-        return  int( self.config['size_gb'] * GIGABYTE)
+    def max_size_bytes(self) -> int:
+        return int(self.config["size_gb"] * GIGABYTE)
 
     @max_size_bytes.setter
-    def max_size_bytes(self,size_bytes:int):
-        self._update_config('size_gb',size_bytes / GIGABYTE)
+    def max_size_bytes(self, size_bytes: int):
+        self._update_config("size_gb", size_bytes / GIGABYTE)
 
     @property
-    def parallel(self)->bool:
-        return self.config['parallel']
+    def parallel(self) -> bool:
+        return self.config["parallel"]
 
     @parallel.setter
-    def parallel(self, parallel:bool):
-        self._update_config('parallel',parallel)
+    def parallel(self, parallel: bool):
+        self._update_config("parallel", parallel)
 
     @property
-    def allow_for_missing_files(self)->bool:
-        return self.config['allow_for_missing_files']
+    def allow_for_missing_files(self) -> bool:
+        return self.config["allow_for_missing_files"]
 
     @allow_for_missing_files.setter
-    def allow_for_missing_files(self, allow_for_missing_files:bool):
-        self._update_config('allow_for_missing_files',allow_for_missing_files)
+    def allow_for_missing_files(self, allow_for_missing_files: bool):
+        self._update_config("allow_for_missing_files", allow_for_missing_files)
 
     def set_directive_function(
-            self,
-            directive,
-            name,
-            function: Union[
-                Callable[[str],None],
-                Callable[[str],bool]
-            ]
-        ):
+        self,
+        directive,
+        name,
+        function: Union[Callable[[str], None], Callable[[str], bool]],
+    ):
 
         if directive not in self.directives:
-            raise KeyError(f'{directive} is not a valid cache directive.')
+            raise KeyError(f"{directive} is not a valid cache directive.")
 
         if name in self.directives[directive]:
-            raise ValueError(f'Function  for {name} already exists')
+            raise ValueError(f"Function  for {name} already exists")
         else:
             self.directives[directive][name] = function
 
-
-    def remove_directive_function(self,directive:str,name:str):
+    def remove_directive_function(self, directive: str, name: str):
 
         if directive not in self.directives:
-            raise KeyError(f'{directive} is not a valid cache directive.')
+            raise KeyError(f"{directive} is not a valid cache directive.")
 
         if name not in self.directives[directive]:
-            raise ValueError(f'Function  for {name} does not exist')
+            raise ValueError(f"Function  for {name} does not exist")
         else:
             self.directives[directive].pop(name)
 
-
     def _cache_file_name(self, uri: str) -> str:
         """
         Return the filename that corresponds to the given uri. We construct
@@ -240,8 +241,7 @@ class FileCache:
         :param uri: valid uri stripped from directives
         :return: valid cache file
         """
-        return self.CACHE_FILE_PREFIX + _hashname(
-            uri) + self.CACHE_FILE_POSTFIX
+        return self.CACHE_FILE_PREFIX + _hashname(uri) + self.CACHE_FILE_POSTFIX
 
     def _cache_file_path(self, uri: str) -> str:
         """
@@ -257,14 +257,16 @@ class FileCache:
         Find all files that are currently a member of the cache.
         :return:
         """
-        _cache_files = []
         for path, dirs, files in os.walk(self.path):
             # Return all files that are "cache" objects. This is a safety if
             # other user files are present, so that these don't accidentally
             # evicted from the cache (aka deleted).
-            return [file for file in files if
-                    file.startswith(self.CACHE_FILE_PREFIX) and
-                    file.endswith(self.CACHE_FILE_POSTFIX)]
+            return [
+                file
+                for file in files
+                if file.startswith(self.CACHE_FILE_PREFIX)
+                and file.endswith(self.CACHE_FILE_POSTFIX)
+            ]
         else:
             return []
 
@@ -287,12 +289,14 @@ class FileCache:
             self._cache_eviction()
         else:
             if self._size() > self.max_size_bytes:
-                raise ValueError('The cache currently existing on disk '
-                                 'exceeds the maximum cache size of the '
-                                 f'current cache ({self.max_size} gb).'
-                                 f'\n The cache size can be increased by'
-                                 f' editting the cache config file: '
-                                 f'{self.config_name}')
+                raise ValueError(
+                    "The cache currently existing on disk "
+                    "exceeds the maximum cache size of the "
+                    f"current cache ({self.max_size} gb)."
+                    f"\n The cache size can be increased by"
+                    f" editting the cache config file: "
+                    f"{self.config_name}"
+                )
 
     def in_cache(self, unparsed_uris) -> List[bool]:
         # make sure input is a list
@@ -327,7 +331,7 @@ class FileCache:
         uri, _ = parse_directive(unparsed_uri)
 
         if not self.in_cache(uri):
-            raise ValueError(f'Key {uri} not in Cache')
+            raise ValueError(f"Key {uri} not in Cache")
 
         _hash = self._cache_file_name(uri)
         return self._remove_item_from_cache(_hash)
@@ -349,12 +353,12 @@ class FileCache:
         file_to_delete = self._entries.pop(_hash)
 
         if os.path.exists(file_to_delete):
-            logger.debug(f' - removing {_hash}')
+            logger.debug(f" - removing {_hash}")
 
             # And delete file.
             os.remove(file_to_delete)
         else:
-            logger.debug(f' - file {_hash} did not exist on disk')
+            logger.debug(f" - file {_hash} did not exist on disk")
 
         return None
 
@@ -368,17 +372,18 @@ class FileCache:
         filepath = self._entries[_hash]
 
         if not os.path.exists(filepath):
-            raise FileNotFoundError('The filepath in the cache log does not'
-                                    'exist on disk.')
+            raise FileNotFoundError(
+                "The filepath in the cache log does not" "exist on disk."
+            )
 
         # Touch the file to indicate we recently used it.
         Path(filepath).touch()
 
         return filepath
 
-    def get_cache_misses(self, uris: List[str],
-                         directives: List[Dict[str, str]]) \
-            -> List[CacheMiss]:
+    def get_cache_misses(
+        self, uris: List[str], directives: List[Dict[str, str]]
+    ) -> List[CacheMiss]:
         """
         Function to get all cache misses and return a list of CacheMiss objects
         needed to download the misses from remote resources.
@@ -403,12 +408,17 @@ class FileCache:
             valid_entry = False
             if self._is_in_cache(hashkey):
                 # If so is it a valid entry
-                if 'validate' in directive:
+                if "validate" in directive:
                     # Call the user supplied validation function with the
                     # filepath as argument
-                    validation_function = \
-                        self.directives['validate'][directive['validate']]
-                    valid_entry = validation_function(filepath)
+                    validation_function = self.directives["validate"][
+                        directive["validate"]
+                    ]
+
+                    try:
+                        valid_entry = validation_function(filepath)
+                    except IOError:
+                        valid_entry = False
 
                     if not valid_entry:
                         # remove the locally stored entry if not valid
@@ -422,11 +432,11 @@ class FileCache:
             if not valid_entry:
                 # If not a valid entry (either missing or invalid)
                 #
-                if 'postprocess' in directive:
+                if "postprocess" in directive:
                     # Add the postprocess function to use if requested.
-                    post_process_function = \
-                        self.directives[
-                            'postprocess'][directive['postprocess']]
+                    post_process_function = self.directives["postprocess"][
+                        directive["postprocess"]
+                    ]
 
                 else:
                     # otherwise set a null function as postprocessor
@@ -438,7 +448,7 @@ class FileCache:
                         filepath=filepath,
                         filename=hashkey,
                         allow_for_missing_files=self.allow_for_missing_files,
-                        post_process_function=post_process_function
+                        post_process_function=post_process_function,
                     )
                 )
         return cache_misses
@@ -486,20 +496,21 @@ class FileCache:
                 cache_misses,
                 self.resources,
                 parallel_download=self.parallel,
-                disable_progress_bar=self.disable_progress_bar
+                disable_progress_bar=self.disable_progress_bar,
             )
 
             for cache_miss in cache_misses:
-                self._add_to_cache(cache_miss.filename,
-                                   cache_miss.filepath)
+                self._add_to_cache(cache_miss.filename, cache_miss.filepath)
 
         size_of_requested_data = _get_total_size_of_files_in_bytes(filepaths)
         if size_of_requested_data > self.max_size_bytes:
-            warning = f'The requested data does not fit into the cache.' \
-                      f'To avoid issues the cache is enlarged to ensure' \
-                      f'the current set of files fits in the cache. \n' \
-                      f'old size: {self.max_size_bytes} bytes; ' \
-                      f'new size {size_of_requested_data + MEGABYTE}'
+            warning = (
+                f"The requested data does not fit into the cache."
+                f"To avoid issues the cache is enlarged to ensure"
+                f"the current set of files fits in the cache. \n"
+                f"old size: {self.max_size_bytes} bytes; "
+                f"new size {size_of_requested_data + MEGABYTE}"
+            )
             warn(warning)
             logger.warning(warning)
             self.max_size_bytes = size_of_requested_data + MEGABYTE
@@ -539,21 +550,24 @@ class FileCache:
             modified_time = os.path.getmtime(fp)
 
             # pick whichever is most recent.
-            time_to_check = access_time if access_time > modified_time \
-                else modified_time
+            time_to_check = (
+                access_time if access_time > modified_time else modified_time
+            )
             modified.append((time_to_check, _hash))
 
         # Sort files in reversed chronological order.
-        files_in_cache = [x[1] for x in
-                          sorted(modified, key=lambda x: x[0], reverse=True)]
+        files_in_cache = [
+            x[1] for x in sorted(modified, key=lambda x: x[0], reverse=True)
+        ]
 
         # Delete files one by one as long as the cache_size exceeds the max
         # size.
         while (_size := self._size()) > self.max_size_bytes:
             self._cache_evictions += 1
             logger.debug(
-                f'Cache exceeds limits: {_size} bytes, max size: '
-                f'{self.max_size_bytes} bytes')
+                f"Cache exceeds limits: {_size} bytes, max size: "
+                f"{self.max_size_bytes} bytes"
+            )
 
             # Get the hash and path of the oldest file and remove
             self._remove_item_from_cache(files_in_cache.pop())
@@ -566,27 +580,29 @@ class FileCache:
         :return: cache size in bytes.
         """
         return _get_total_size_of_files_in_bytes(
-            list(self._entries.values()), self.path)
+            list(self._entries.values()), self.path
+        )
 
     def purge(self) -> None:
         """
         Delete all the files in the cache.
         :return: None
         """
-        logger.debug(f'Purging cache')
+        logger.debug("Purging cache")
         keys = list(self._entries.keys())
         for key in keys:
             filepath = self._entries.pop(key)
-            logger.debug(f' - deleting {filepath}')
+            logger.debug(" - deleting {filepath}")
             os.remove(filepath)
-        logger.debug(f'Purging cache done')
+        logger.debug("Purging cache done")
 
 
-def _download_from_resources(cache_misses: List[CacheMiss],
-                             resources: List[RemoteResource],
-                             parallel_download=False,
-                             disable_progress_bar = False,
-                             ) -> List[bool]:
+def _download_from_resources(
+    cache_misses: List[CacheMiss],
+    resources: List[RemoteResource],
+    parallel_download=False,
+    disable_progress_bar=False,
+) -> List[bool]:
     """
     Wrapper function to download multiple uris from the resource(s).
     :param cache_misses: List containing cache misses to download
@@ -601,7 +617,7 @@ def _download_from_resources(cache_misses: List[CacheMiss],
             return True
         except _RemoteResourceUriNotFound as e:
             if cache_miss.allow_for_missing_files:
-                warning = f'Uri not retrieved: {str(e)}'
+                warning = f"Uri not retrieved: {str(e)}"
                 warn(warning)
                 logger.warning(warning)
             else:
@@ -625,17 +641,13 @@ def _download_from_resources(cache_misses: List[CacheMiss],
         else:
             # If we didn't break the loop no valid resource was found, raise
             # error
-            raise ValueError(f'No resource available for URI: '
-                             f'{cache_miss.uri}')
+            raise ValueError(f"No resource available for URI: " f"{cache_miss.uri}")
 
     # Download the requested objects.
     if parallel_download and len(cache_misses) > 1:
         with ThreadPool(processes=MAXIMUM_NUMBER_OF_WORKERS) as pool:
             output = list(
-                tqdm(
-                    pool.imap(_worker, cache_misses),
-                    total=len(cache_misses)
-                )
+                tqdm(pool.imap(_worker, cache_misses), total=len(cache_misses))
             )
     else:
         if len(cache_misses) == 1:
@@ -644,7 +656,8 @@ def _download_from_resources(cache_misses: List[CacheMiss],
         output = list(
             tqdm(
                 map(_worker, cache_misses),
-                total=len(cache_misses),disable=disable_progress_bar
+                total=len(cache_misses),
+                disable=disable_progress_bar,
             )
         )
     return output
@@ -720,26 +733,26 @@ def parse_directive(unparsed_uri: str) -> Tuple[str, dict]:
     """
 
     # split in directives_scheme part and a path.
-    directives_and_scheme, path = unparsed_uri.split('://')
+    directives_and_scheme, path = unparsed_uri.split("://")
 
     parsed_directives = {}
     # if a colon is present then directives are provided.
-    if ':' in directives_and_scheme:
+    if ":" in directives_and_scheme:
         # split directives from the scheme
-        directives, scheme = directives_and_scheme.split(':')
+        directives, scheme = directives_and_scheme.split(":")
 
         # split multiple directives (if present)
-        directives = directives.split(';')
+        directives = directives.split(";")
 
         # for each directive store in the dict.
         for directive in directives:
-            directive_name, directive_parameter = directive.split('=')
+            directive_name, directive_parameter = directive.split("=")
             parsed_directives[directive_name] = directive_parameter
     else:
         # no directives
         scheme = directives_and_scheme
 
-    uri = scheme + '://' + path
+    uri = scheme + "://" + path
     return uri, parsed_directives
 
 
